#region File Header
// <copyright company="Bosch Security Systems GmbH">
//
// Authors:
//    Hubert Streidl<Hubert.Streidl@de.bosch.com>
//    Andreas Wetterich
//
// $Revision: 1.6 $
//
// </copyright>
#endregion

using System;
using System.Text;
using System.IO;
using System.Reflection;

//the api version used in this sample
using FSM5000FSIAPI.Version3;

namespace FSISample5
{
   class Configurator
   {
      static int theInstance = 0;

      FSIConfig cfgFSI;

      public FSIConfig Configuration
      {
         get { return cfgFSI; }
      }

      public string SessionName
      {
         get {
                  StringBuilder stringB = new StringBuilder();

                  stringB.Append(DateTime.Now.ToShortDateString());
                  stringB.Append(" ");
                  stringB.Append(DateTime.Now.Hour.ToString());
                  stringB.Append("-");
                  stringB.Append(DateTime.Now.Minute.ToString());
                  stringB.Append(" Node ");
                  stringB.Append(this.cfgFSI.MPNetGroup.ToString());
                  stringB.Append("-");
                  stringB.Append(this.cfgFSI.MPNetNode.ToString());

                  return stringB.ToString();
            }
      }

      public Configurator()
      {
         cfgFSI = new FSIConfig();
         cfgFSI.LocalIPAddress = new byte[4];
         cfgFSI.AdvancedFSIConfig = new AdvancedFSIConfig();
         cfgFSI.AdvancedFSIConfig.MulticastAddress = new byte[4];
         theInstance++;


         // instance 1:                instance2:        instance3:        instance4:
         // PNA = 33                   PNA = 34          PNA = 35          PNA = 36
         // Group = 2                  Group = 2         Group = 2         Group = 2
         // Node = 1                   Node = 2          Node = 3          Node = 4
         // LocalIP = 192.168.1.33     LocalIP = *.34    LocalIP = *.35    LocalIP = *.36
         // *** adapt defaults according to your own configuration here or at runtime ***
         cfgFSI.PNA = (byte) (32 + theInstance);
         cfgFSI.MPNetGroup = 2;
         cfgFSI.MPNetNode = (byte) theInstance;
         cfgFSI.LocalIPAddress[0] = 192;
         cfgFSI.LocalIPAddress[1] = 168;
         cfgFSI.LocalIPAddress[2] = 1;
         cfgFSI.LocalIPAddress[3] = (byte) (32 + theInstance);

         cfgFSI.AdvancedFSIConfig.PortNumber = 25001;
         cfgFSI.AdvancedFSIConfig.MulticastAddress[0] = 239;
         cfgFSI.AdvancedFSIConfig.MulticastAddress[1] = 192;
         cfgFSI.AdvancedFSIConfig.MulticastAddress[2] = 0;
         cfgFSI.AdvancedFSIConfig.MulticastAddress[3] = 1;
      }

      public void ReadFromConsole()
      {
         //Group number
         string output = string.Format("Enter Group Number ( {0} ): ", this.cfgFSI.MPNetGroup);
         Console.WriteLine(output);
         this.cfgFSI.MPNetGroup = ReadByteFromConsole(this.cfgFSI.MPNetGroup);

         //Node number
         output = string.Format("Enter Node Number ( {0} ): ", this.cfgFSI.MPNetNode);
         Console.WriteLine(output);
         this.cfgFSI.MPNetNode = ReadByteFromConsole(this.cfgFSI.MPNetNode);

         //PNA
         output = string.Format("Enter PNA ( {0} ): ", this.cfgFSI.PNA);
         Console.WriteLine(output);
         this.cfgFSI.PNA = ReadByteFromConsole(this.cfgFSI.PNA);

         //IP address
         output = string.Format("Enter IP.Address( {0}.{1}.{2}.{3} ): ", this.cfgFSI.LocalIPAddress[0], this.cfgFSI.LocalIPAddress[1], 
                                                                       this.cfgFSI.LocalIPAddress[2], this.cfgFSI.LocalIPAddress[3] );
         Console.WriteLine(output);
         ReadAddressFromConsole(this.cfgFSI.LocalIPAddress);

         output = string.Format("Enter port number: {0}: ", this.cfgFSI.AdvancedFSIConfig.PortNumber);
         Console.WriteLine(output);
         this.cfgFSI.AdvancedFSIConfig.PortNumber = ReadIntegerFromConsole(this.cfgFSI.AdvancedFSIConfig.PortNumber);

         output = string.Format("Enter multicast address( {0}.{1}.{2}.{3} ): ", this.cfgFSI.AdvancedFSIConfig.MulticastAddress[0], this.cfgFSI.AdvancedFSIConfig.MulticastAddress[1], 
                                                                              this.cfgFSI.AdvancedFSIConfig.MulticastAddress[2], this.cfgFSI.AdvancedFSIConfig.MulticastAddress[3] );

         Console.WriteLine(output);
         ReadAddressFromConsole(this.cfgFSI.AdvancedFSIConfig.MulticastAddress);

         this.cfgFSI.RepositoryLocation = CreateRepositoryFolderName();

         Console.WriteLine();
         PrintConfigurationToConsole();
      }


      public byte ReadByteFromConsole(byte inputByte)
      {
         byte value = inputByte;

         string line = Console.ReadLine();

         if (false == byte.TryParse(line, out value))
         {
            value = inputByte;
         }

         return value;
      }

      public ushort ReadIntegerFromConsole(ushort input)
      {
         ushort value = input;

         string line = Console.ReadLine();

         if (false == ushort.TryParse(line, out value))
         {
            value = input;
         }

         return value;
      }

      public void ReadAddressFromConsole(byte[] address)
      {
         Byte[] bytes = address;

         string line = Console.ReadLine();

         bool bParseSuccess = true;
         // split into subStrings for each ip field
         String[] ipList = line.Split('.');
         if (ipList.Length == 4)
         {
            for (int i = 0; i < ipList.Length; i++)
            {
               Byte b;
               if (true == Byte.TryParse(ipList[i], out b))
               {
                  bytes[i] = b;
               }
               else
               {
                  bParseSuccess = false;
                  break;
               }
            }
         }
         else
         {
            bParseSuccess = true;
         }

         if (true == bParseSuccess)
         {
            for (int i = 0; i < bytes.Length; i++)
            {
               address[i] = bytes[i];
            }
         }
         else
         {
            //Console.WriteLine("Invalid input - kept previous setting");
         }
      }

      public void PrintConfigurationToConsole()
      {
         //LNA
         string output = string.Format("Logical Node Address: {0}.{1}: ", this.cfgFSI.MPNetGroup, this.cfgFSI.MPNetNode);
         Console.WriteLine(output);

         //PNA
         output = string.Format("Physical Node Address ({0}): ", this.cfgFSI.PNA);
         Console.WriteLine(output);

         //IP address
         output = string.Format("IP.Address: {0}.{1}.{2}.{3} ", this.cfgFSI.LocalIPAddress[0], this.cfgFSI.LocalIPAddress[1], 
                                                                this.cfgFSI.LocalIPAddress[2], this.cfgFSI.LocalIPAddress[3]);
         Console.WriteLine(output);

         output = string.Format("Port number: {0}: ", this.cfgFSI.AdvancedFSIConfig.PortNumber);
         Console.WriteLine(output);

         output = string.Format("Multicast address: {0}.{1}.{2}.{3}: ", this.cfgFSI.AdvancedFSIConfig.MulticastAddress[0], this.cfgFSI.AdvancedFSIConfig.MulticastAddress[1],
                                                                      this.cfgFSI.AdvancedFSIConfig.MulticastAddress[2], this.cfgFSI.AdvancedFSIConfig.MulticastAddress[3]);
         Console.WriteLine(output);
      }



      private string CreateRepositoryFolderName()
      {
         StringBuilder stringB = new StringBuilder();

         // get entrance location of the assembly
         FileInfo fi = new FileInfo(Assembly.GetEntryAssembly().Location);

         // return directory path
         stringB.Append(fi.DirectoryName);
         stringB.Append(@"\Repository-");
         stringB.Append(this.cfgFSI.MPNetGroup.ToString());
         stringB.Append("-");
         stringB.Append(this.cfgFSI.MPNetNode.ToString());

         return stringB.ToString();
      }
   }

   
}
